"""Top-level package for Tank Forecaster."""

__author__ = """Jim Vogel"""
__email__ = "james.vogel@capspire.com"
__version__ = "0.1.35"

from tank_forecaster.tank_forecaster import forecast_near, forecast_far, clean
